--
-- MapBGABunker
-- This is the specialization for MapBGABunker
--
-- M@D Author  Headshot XXL
-- M@D date 23.07.2010
--
-- > Copyright (C) Headshot XXL - www.planet-ls.de < --


print("Map: MapBGABunker v1.0 loaded");

if BaseMission.modMapShovelTrigger == nil then
	BaseMission.modMapShovelTrigger = {};
	print("Register Variable: BaseMission.modMapShovelTrigger");
end;

MapBGABunker = {};

_G.BGABunkerOnCreate = function(id)
	--print("created BGASilo trigger, id: ", id);
	MapBGABunker:load(id)
end;

function MapBGABunker:load(id)

	addTrigger(id, "BunkerCallback", self);
	
	self.moveMaxY = getUserAttribute(id, "moveMaxY");
	self.moveMinY = getUserAttribute(id, "moveMinY");
	self.capacity = getUserAttribute(id, "capacity");
	self.electricityPrice = getUserAttribute(id, "electricityPrice");
	self.power = getUserAttribute(id, "power");
	self.siloPlane = Utils.indexToObject(getParent(id), getUserAttribute(id, "movingIndex"));
	
	self.allowFillTypes = {};
	local types = Utils.splitString(" ", getUserAttribute(id, "fruitTypes"));
	for k,v in pairs(types) do
		if FruitUtil.fruitTypes[v] ~= nil then
			self.allowFillTypes[FruitUtil.fruitTypes[v].index] = true;
		end;
	end;
	
	self.emptyNode = id;

	self.fillLevel = 0;
	self.money = 0;
	self.yesterDayMoney = 0;

	
	table.insert(g_currentMission.modMapShovelTrigger, MapBGABunker);

	
	self.manure = {};
	local manureTrigger = Utils.indexToObject(getParent(id), getUserAttribute(id, "manureTriggerIndex"));
	self.manure.moveMaxY = getUserAttribute(manureTrigger, "moveMaxY");
	self.manure.moveMinY = getUserAttribute(manureTrigger, "moveMinY");
	self.manure.capacity = getUserAttribute(manureTrigger, "capacity");
	self.manure.manurePlane = Utils.indexToObject(getParent(manureTrigger), getUserAttribute(manureTrigger, "movingIndex"));	
	self.manure.fillSpeed = getUserAttribute(manureTrigger, "fillSpeed");
	local fillType = getUserAttribute(manureTrigger, "fillType");		
	local key = "FRUITTYPE_"..string.upper(fillType);
	self.manure.fillType = FruitUtil[key];
	
	addTrigger(manureTrigger, "manureTriggerCallback", self);

	self.manure.trailerInTrigger = nil;	
	self.manure.fillLevel = 0;
	
	self.manure.fill = false;
	self.manure.printText = false;
	
	
	local dir = getUserProfileAppPath() .. "savegame"..gameMenuSystem.quickPlayMenu.selectedIndex;
	createFolder(dir);
    self.saveXMLFile = dir .. "/bgaBunker.xml";

	if g_missionLoaderDesc.stats.fuelUsage == 0 then
		os.remove(self.saveXMLFile);
	else
		MapBGABunker:loadBGAFromFile();
	end;
	
end;

function MapBGABunker:loadMap(name)
end;

function MapBGABunker:deleteMap()
end;

function MapBGABunker:delete()
end;

function MapBGABunker:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MapBGABunker:keyEvent(unicode, sym, modifier, isDown)
end;

function MapBGABunker:update(dt)

	if self.fillLevel > 0 and self.manure.fillLevel < self.manure.capacity then
		local LPerKWPerHour = 2.29; --20t/1kw/365/24 #rounded
		local level = self.power * (LPerKWPerHour*(dt*g_currentMission.environment.timeScale)/(1000*60*60));
		self.fillLevel = self.fillLevel - level;
		self.manure.fillLevel = self.manure.fillLevel + level;
		self.money = self.money + (self.electricityPrice*self.power)*((dt*g_currentMission.environment.timeScale)/(1000*60*60)); -- 1h=1000*60*60
	end;
	
	if g_currentMission.environment.dayTime == 0 then
		g_currentMission.missionStats.money = g_currentMission.missionStats.money + self.money;
		self.yesterDayMoney = self.money;
		self.money = 0;
	end;

	if self.fillLevel > self.capacity+1000 then
		self.fillLevel = self.capacity+1000;
	end;
	
	if self.fillLevel > 0 then
		--local m = (self.moveMaxY - self.moveMinY) / self.capacity;
		--local xPos, yPos, zPos = getTranslation(self.siloPlane);
		--setTranslation(self.siloPlane, xPos, m*self.fillLevel + self.moveMinY, zPos);
		self:setFillLevel(self.fillLevel, nil);
	elseif self.fillLevel < 0 then
		self.fillLevel = 0;
	end;
	if self.manure.fillLevel > 0 then
		local m = (self.manure.moveMaxY - self.manure.moveMinY) / self.manure.capacity;
		local xPos, yPos, zPos = getTranslation(self.manure.manurePlane);
		setTranslation(self.manure.manurePlane, xPos, m*self.manure.fillLevel + self.manure.moveMinY, zPos);
	end;
	
	
	if self.manure.trailerInTrigger ~= nil and self.manure.fillLevel > 0 then	
		
		if InputBinding.hasEvent(InputBinding.BGAMANURE_FILL)  then
			self.manure.fill = not self.manure.fill; 
		end; 
		if self.manure.fill and self.manure.printText then
			g_currentMission:addExtraPrintText(g_i18n:getText("BGAMANURE_1").. " " ..InputBinding.getButtonKeyName(InputBinding.BGAMANURE_FILL)); 
		elseif self.manure.printText then
			g_currentMission:addExtraPrintText(g_i18n:getText("BGAMANURE_2").. " " ..InputBinding.getButtonKeyName(InputBinding.BGAMANURE_FILL)); 
		end; 
		
		self.manure.printText = false;				
		local attachable = self.manure.trailerInTrigger;
		if attachable.fillTypes ~= nil and attachable.setFillLevel ~= nil and attachable.fillLevel ~= nil then
			if attachable.fillLevel < attachable.capacity and attachable:allowFillType(self.manure.fillType, true) then
				if self.manure.fill then
					local deltaFillLevel = dt*self.manure.fillSpeed;
					attachable:setFillLevel(attachable.fillLevel+deltaFillLevel, self.manure.fillType);
					self.manure.fillLevel = self.manure.fillLevel - deltaFillLevel;
					--local fruitDesc = FruitUtil.fruitIndexToDesc[self.manure.fruitTrigger[i].fillType];
					--local price = fruitDesc.seedPricePerLiter*deltaFillLevel;
					--g_currentMission.missionStats.money = g_currentMission.missionStats.money - price*2*g_currentMission.missionStats.difficulty;
				end;
				self.manure.printText = true;
			else
				self.manure.fill = false;
			end;
		end;		
	else
		self.manure.fill = false;
	end;
	
end;

function MapBGABunker:draw()
end;

function MapBGABunker:setFillLevel(fillLevel, fillType)

	self.fillLevel = fillLevel;
	
	local m = (self.moveMaxY - self.moveMinY) / self.capacity;
	local xPos, yPos, zPos = getTranslation(self.siloPlane);
	setTranslation(self.siloPlane, xPos, m*self.fillLevel + self.moveMinY, zPos);

end;

function MapBGABunker:BunkerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter then
		if otherId ~= 0 then
			local isStrawbale = getUserAttribute(otherId, "isStrawbale");
			local isHaybale = getUserAttribute(otherId, "isHaybale");
			local isSilobale = getUserAttribute(otherId, "isSilobale");

			if (isStrawbale ~= nil and isStrawbale) or (isHaybale ~= nil and isHaybale) or (isSilobale ~= nil and isSilobale) then
				--print("Bale stored: ", tostring(otherId));
				self.fillLevel = self.fillLevel + 3000;
				g_currentMission:removeItemToSave(otherId);
				delete(otherId);
			end;			
		end;
	end;
end;

function MapBGABunker:manureTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)

    if onEnter then
		if otherId ~= nil then		
			local trailer = g_currentMission.objectToTrailer[otherId];
			if trailer ~= nil then
				if trailer.setFillLevel ~= nil and trailer:allowFillType(self.manure.fillType, true) then	
				self.manure.trailerInTrigger = trailer;
				end;
			end;
		end;		
	elseif onLeave then
		if otherId ~= nil then		
			local trailer = g_currentMission.objectToTrailer[otherId];
			if trailer ~= nil then
				self.manure.trailerInTrigger = nil;
			end;
		end;
	end;

end;

function MapBGABunker:allowFillType(fillType, allowEmptying)
    local allowed = false;

    if self.allowFillTypes[fillType] then
        allowed = true;
    end;

    return allowed;
end;


function MapBGABunker:saveBGAToXML()	
	
	local existDir = io.open (self.saveXMLFile, "w");	
	if existDir == nil then
		createXMLFile("createsilo", self.saveXMLFile, "savesilo");
	end;

    -- save silo filllevel
    local xmlFile = io.open (self.saveXMLFile, "w");
    if xmlFile ~= nil then
        xmlFile:write('<?xml version="1.0" encoding="iso-8859-1" standalone="no" ?>\n<bga>\n');

		xmlFile:write('    <bunker bunkerFillLevel="'..self.fillLevel..'" manureFillLevel="'..self.manure.fillLevel..'" money="'..self.money..'" yesterDayMoney="'..self.yesterDayMoney..'" />\n');

        xmlFile:write("</bga>");

        xmlFile:close();
    end;	
	
end;

local QuickPlayMenuSaveSelectedGame = QuickPlayMenu.saveSelectedGame;
QuickPlayMenu.saveSelectedGame = function(self)
	QuickPlayMenuSaveSelectedGame(self); 
	MapBGABunker:saveBGAToXML();
	--print("bga saved")
end;

function MapBGABunker:loadBGAFromFile()

	local existDir = io.open (self.saveXMLFile, "r");
	if existDir ~= nil then
		local xmlFile = loadXMLFile("TempConfig", self.saveXMLFile);
	
		local siloI = 0;
		local key = string.format("bga.bunker(%d)", siloI);
		if key ~= nil then
			self.fillLevel = getXMLFloat(xmlFile, key.."#bunkerFillLevel");
			self.manure.fillLevel = getXMLFloat(xmlFile, key.."#manureFillLevel");
			self.money = getXMLFloat(xmlFile, key.."#money");
			self.yesterDayMoney = getXMLFloat(xmlFile, key.."#yesterDayMoney");
		end;
		
		siloI = siloI +1;
		
		delete(xmlFile);
		print("BGA Bunker saves loaded");
	else
		print("Warning: ", self.saveXMLFile, " can not be found, BGA Bunker saves not loaded.")
	end;

end;

addModEventListener(MapBGABunker);

